/*******************************************************
 
Author : Mauro Laurenti
Version : 1.0
Date : 25/5/2006
 
CopyRight 2007 all rights are reserved


********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) 
is intended for use solely and exclusively on Microchip PIC Microcontroller (registred mark).  
The software is owned by the Author, and is protected under applicable copyright laws. 
All rights are reserved. 
Any use in violation of the foregoing restrictions may subject the 
user to criminal sanctions under applicable laws (Italian or International ones), as well as to 
civil liability for the breach of the terms and conditions of this license. 
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following 
software doesn't aim to commercial use or monetary earning of any kind.    

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT, 
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************
 
 
Metro ad ultrasuoni
 
Visita www.LaurTec.com per ulteriori informazioni 

*******************************************************/


#include <p18f4580.h>
#include <timers.h>
#include <stdlib.h>
#include <ctype.h>
#include "LCD_44780_Freedom.h"	//bisogna scaricare la libreria C18 sul sito www.LaurTec.com
#include "Sponsor.h" 

#pragma config OSC = HS 	// 20Mhz
#pragma config WDT = OFF	// disattivo il watchdog timer
#pragma config LVP = OFF	// disattivo la programmazione LVP
#pragma config PBADEN = OFF	// disabilito gli ingressi analogici sulla PORTB

#define Trig PORTCbits.RC0
#define Echo PORTCbits.RC5

void Int_Event (void);		// prototipo di funzione
void delay (int);

int total_us ;	// variabili globali per la misura dello spazio
int distance;	// variabile che contiene la distanza come valore intero in cm
unsigned char distanceLCD [3]; //come sopra ma il valore e' una stringa visualizzabile su LCD


#pragma code low_vector=0x18

void low_interrupt (void)
{
  _asm GOTO Int_Event _endasm	//imposta il salto per la gestione dell'interrupt
}

#pragma code



#pragma interruptlow Int_Event


void Int_Event (void)
{	static unsigned char num =0;	// variabile per il conteggio degli interrupt
	
	
	
	if (INTCONbits.TMR0IF == 1 )	// Controllo che l'interrupt sia stato generato da TMR0 
	{		
 		INTCONbits.TMR0IF = 0;
  		num++;
  		
  		if (num == 20 || num == 40 || num == 60 || num == 80 || num == 100 || num == 120 || num == 140 || num == 160 || num == 180 || num == 200|| num == 220 || num == 240 || num == 255)
  		{	Trig = 1;		//invio dell'impulso di Trig
  			delay (50);
  			Trig = 0;
  		}

	}
	
}



void main (void)
{	
	TRISA = 0xFF;
	PORTA = 0x00;
	 
	TRISB = 0x00 ;	
	PORTB = 0x00 ;
	
	TRISC = 0xFE;  // inizializzo la PORTC per l'utilizzo del sensore ad ultrasuoni
	PORTC = 0x00;
	
	TRISD = 0x00;	// la PORTD  impostata per funzionare correttamente con l'LCD	
	PORTD = 0x00;
	
	TRISE = 0x00;
	PORTE = 0x00;
	
	OpenLCD ();
	
	WriteSponsor ();
	WriteStringLCD ("Distanza: --- cm");
			
	
//sposto il cursore all'inizio della cifra piu' significativa della misura	
	ShiftCursorLCD (LEFT);
	ShiftCursorLCD (LEFT);
	ShiftCursorLCD (LEFT);
	ShiftCursorLCD (LEFT);
	ShiftCursorLCD (LEFT);
	ShiftCursorLCD (LEFT);


  	OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_16BIT );
  	  
  	INTCONbits.GIE = 1;		//	Enable global interrupts. 
	
	
	while (1)
	{
		
	
		if (Echo == 1)
			{	
				total_us = 0;
				
				while (Echo == 1)
				{
					total_us++;
					
					_asm
					nop			//il nop rallenta il ciclo di lettura tarando correttamente la lettura
					_endasm
							
					
				}	
				
				distance = total_us >>5;	//divisione per 32
				
// oltre i 4 metri la misura perde di significato e viene visualizzato <->
				
				if (distance>400)	
					{	
						distanceLCD[2]='>';
						distanceLCD[1]='-';
						distanceLCD[0]='<';						
					}
				else 
					{

						itoa (distance,distanceLCD);	// converto la distanza in una stringa	

//sposto le cifre in modo da mantenere lo stesso valore posizionale				
						if (!isdigit(distanceLCD[1]))	
							{
								distanceLCD[2]=distanceLCD[0];
								distanceLCD[1]=' ';
								distanceLCD[0]=' ';	
							}
		
						if (!isdigit(distanceLCD[2]))
							{	
								distanceLCD[2]=distanceLCD[1];
								distanceLCD[1]= distanceLCD[0];
								distanceLCD[0]=' ';
						
							}
					}				
							
				WriteCharLCD (distanceLCD[0]);	//scrivo la distanza sul Display
				WriteCharLCD (distanceLCD[1]);
				WriteCharLCD (distanceLCD[2]);
				
				ShiftCursorLCD (LEFT);		//riposiziono il cursore in modo da aggiornare
				ShiftCursorLCD (LEFT);		//solo le cifre di interesse
				ShiftCursorLCD (LEFT);
				
							
			}
			
		// ulteriori controlli nel programma vanno inseriti dopo questo testo
				
	} 
}


//*********************************************************************************************
//this function accept in input the number of us for the dalay
//the number of nop inside the loop are trimmed for 20MHz Quartz

void delay (int attesa)
{	int i;
	for (i =0; i<attesa;i++)
		{				
			_asm 
			nop
			nop
			nop
			nop
			_endasm
		}	
}
